# Limita funkce a spojitost

Mějme dánu funkci $f : D \to \mathbb{R}$ a bod $x_0 \in \mathbb{R}^*$, který je hromadným bodem $D$.

Řekneme, že funkce $f$ má **limitu** $b \in \mathbb{R}^*$ v bodě $x_{0}$, jestliže pro **každou** posloupnost $(x_{0})$ platí
$$
\left( ( \space \forall \, n \in \mathbb{N} : x_{n} \in D \quad \land \quad x_{n} \neq x_{0} \space ) \quad \land \quad \lim_{ n \to \infty }{x_{n}} = x_{0} \space \right) \quad \implies \quad \lim_{ n \to \infty }{f(x_{n})} = b 
$$

a píšeme $\displaystyle\lim_{ x \to \infty }{f(x)} = b$.

### Jednoznačnost limity

Každá funkce má v bodě nejvýše jednu limitu (limitu zprava, limitu zleva).

Pro $x_{0} \in \mathbb{R}$ a $b \in \mathbb{R}^*$ platí $\displaystyle \lim_{ x \to x_{0} } f(x) = b$ právě tehdy, když $\displaystyle \lim_{ x \to x_{0}^- } f(x) = \lim_{ x \to x_{0}^+ } f(x) = b$.

### Algebra limit

Mějme dány funkce $f$ a $g$, které mají stejný definiční obor $D$ a mají v bodě $x_{0} \in \mathbb{R}^*$ limitu

$$\lim_{ x \to x_{0} } f(x) = a \in \mathbb{R}^*, \lim_{ x \to x_{0} } g(x) = b \in \mathbb{R}^*.$$

Potom platí
- $\displaystyle\lim_{ x \to x_{0} } (f(x) + g(x)) = a + b, \quad$ pokud je pravá strana definována,
- $\displaystyle\lim_{ x \to x_{0} } (f(x) \cdot g(x)) = a \cdot b, \quad$ pokud je pravá strana definována,
- $\displaystyle\lim_{ x \to x_{0} } \frac{f(x)}{g(x)} = \frac{a}{b}, \quad$ pokud $\quad\forall \, x \in D : g(x) \neq 0\quad$ a pokud je pravá strana definována.

### Věta o sevření

Mějme dány funkce $f, g, h$ se stejným definičním oborem $D$ a bod $x_{0} \in \mathbb{R}^*$. Dále předpokládejme, že platí
1) $\exists \, \delta > 0 \, \forall \, x \in D \, \cap \, P(x_{0}, \delta) : f(x) \leq g(x) \leq h(x)$,
2) $\displaystyle \lim_{ x \to x_{0} } f(x) = \lim_{ x \to x_{0} } h(x) = b \in \mathbb{R}^*$.

Věta 4.5, 4.6

## Spojitost funkce

- spojité funkce umíme načrtnout jedním tahem
- příklad
	- spojité procesy (růst člověka)
	- nespojité procesy (bankovní účet)

Mějme dánu funkci $f : D \to \mathbb{R}$, bod $x_{0} \in D$, který je hromadným bodem $D$. Řekněme, že funkce $f$ je

| typ spojitosti                 | podmínka                                                   | 
| ------------------------------ | ---------------------------------------------------------- |
| spojitá v $x_0 \in D_f$        | pokud $\displaystyle f(x_{0}) = \lim_{ x \to x_{0} } f(x)$ |
| spojitá zprava v $x_0 \in D_f$ | pokud $\displaystyle f(x_{0}) = f(x_{0}+)$                 |
| spojitá zleva v $x_0 \in D_f$  | pokud $\displaystyle f(x_{0}) = f(x_{0}-)$                 |

Pokud $x_{0} \in D$ je izolovaným bodem $D$, potom funkce $f$ je **spojitá v bodě** $x_{0}$.

### Body nespojitosti

Mějme dánu funkci $f : D \to \mathbb{R}$ a bod $x_{0} \in \mathbb{R}$, pro který $\exists \, \delta > 0 : P(x_{0}, \delta) \subset D$.

Bod $x_{0}$ je **bod nespojitosti** funkce $f$, pokud funkce $f$ v bodě $x_{0}$ není spojitá.

**Druhy bodů nespojitosti**:
- **ON** - odstranitelná nespojitost
	- **podmínka**: $\displaystyle f(x_{0}) \neq \lim_{ x \to x_{0} } f(x) \in \mathbb{R}$
	- limita zprava i zleva je stejná: $f(x_{0}+) = f(x_{0}-)$
	- funkční hodnota v $x_0$ se nerovná limitě v $x_0$, která je vlastní
- **NN1D** - neodstranitelné nespojitost 1. druhu
	- **podmínka**: $f(x_{0}+), f(x_{0}-) \in \mathbb{R}$, ale $f(x_{0}+) \neq f(x_{0}-)$
	- limita zprava i zleva je vlastní, ale nerovnají se
	- nazývá se také **skoková nespojitost** se skokem $s = \dots$
- **NN2D** - neodstranitelná nespojitost 2. druhu
	- **podmínka**: neexistuje vlastní limita $f(x_{0}+)$ nebo $f(x_{0}-)$
	- alespoň jedna neexistuje nebo není alespoň jedna vlastní

Věta 4.7, 4.8, 4.9

### Spojitost na intervalu

Mějme dánu funkci $f : D \to \mathbb{R}$ a interval $I \subset D$. Řekněme, že funkce $f$ je **spojitá na intervalu** $I$ jestliže je spojitá v každém vnitřním bodě intervalu $I$ a patří-li levý (pravý) koncový bod tohoto intervalu do $I$, je v něm spojitá zprava (zleva).

#### Cauchyho věta

Mějme dánu funkci $f$, která je spojitá na **uzavřeném** intervalu $\langle a;b \rangle$ a pro kterou platí $f(a) \cdot f(b) < 0$. Potom existuje $\xi \in (a;b)$ tak, že $f(\xi) = 0$.

#### Weierstrassova věta

Mějme dánu funkci $f$, která je spojitá na **uzavřeném** intervalu. Potom funkce $f$ je na tomto intervalu omezená a nabývá na něm své nejmenší a největší funkční hodnoty.

#### Bolzanova věta

Mějme dánu funkci $f$, která je spojitá na **uzavřeném** intervalu. Potom funkce $f$ na tomto intervalu nabývá všech mezihodnot mezi svou nejmenší a největší funkční hodnotou.